import type { APIRoute } from "astro";
import { getS3Data, putS3Data } from "../../lib/s3";
import type { RSVPItem } from "../../lib/types";
import { createProtectedAPIRoute } from "../../utils/auth-middleware";

const objectsToCSV = (data: RSVPItem[]): string => {
  const headers = ["name", "attending", "dietaryRestrictions", "notes", "timestamp"];
  const csvRows = [headers.join(",")];

  data.forEach((entry) => {
    const row = headers.map((header) => {
      let value = entry[header as keyof RSVPItem];
      
      // Ensure proper value formatting
      if (header === "attending") {
        value = Boolean(value).toString();
      } else if (value === null || value === undefined) {
        value = "";
      }
      
      // Escape special characters
      const escaped = String(value)
        .replace(/"/g, '""')
        .replace(/\n/g, ' ');
      return `"${escaped}"`;
    });
    csvRows.push(row.join(","));
  });

  return csvRows.join("\n");
};

// Sanitization helpers
const sanitizeString = (value: unknown): string => {
  if (typeof value !== 'string') return '';
  // Remove any control characters and normalize whitespace
  return value
    .replace(/[\x00-\x1F\x7F-\x9F]/g, '') // Remove control characters
    .replace(/\s+/g, ' ')                   // Normalize whitespace
    .trim();
};

const sanitizeBoolean = (value: unknown): boolean => {
  if (typeof value === 'boolean') return value;
  if (typeof value === 'string') {
    const lowered = value.toLowerCase().trim();
    return !(lowered === 'false' || lowered === '0' || lowered === 'no');
  }
  return Boolean(value);
};

const sanitizeTimestamp = (value: unknown): string => {
  if (typeof value !== 'string') return new Date().toISOString();
  try {
    const date = new Date(value);
    if (isNaN(date.getTime())) throw new Error('Invalid date');
    return date.toISOString();
  } catch {
    return new Date().toISOString();
  }
};

const sanitizeRSVP = (data: Record<string, unknown>): RSVPItem => {
  return {
    name: sanitizeString(data.name),
    attending: sanitizeBoolean(data.attending),
    dietaryRestrictions: sanitizeString(data.dietaryRestrictions),
    notes: sanitizeString(data.notes),
    timestamp: sanitizeTimestamp(data.timestamp),
  };
};

const csvToObjects = (csv: string): RSVPItem[] => {
  const lines = csv.split("\n");
  const headers = lines[0].split(",").map((h) => h.trim());

  return lines
    .slice(1)
    .filter((line) => line.trim())
    .map((line) => {
      // More robust CSV parsing
      const values: string[] = [];
      let inQuote = false;
      let currentValue = '';
      let isEscaped = false;
      
      for (let i = 0; i < line.length; i++) {
        const char = line[i];
        
        if (char === '"') {
          if (inQuote && line[i + 1] === '"') {
            // Handle escaped quotes
            currentValue += '"';
            i++;
            isEscaped = true;
          } else if (!isEscaped || (isEscaped && inQuote)) {
            inQuote = !inQuote;
            isEscaped = false;
          } else {
            currentValue += char;
          }
        } else if (char === ',' && !inQuote) {
          values.push(currentValue.trim());
          currentValue = '';
          isEscaped = false;
        } else {
          currentValue += char;
        }
      }
      values.push(currentValue.trim());

      // Create a data object from the CSV values
      const data: Record<string, unknown> = {};
      headers.forEach((header, index) => {
        if (index < values.length) {
          let value = values[index];
          value = value.replace(/^"(.*)"$/, "$1").replace(/""/g, '"');
          data[header] = value;
        }
      });

      // Sanitize the data
      return sanitizeRSVP(data);
    })
    .filter(entry => entry.name.length > 0); // Only keep entries with non-empty names
};

// GET: Retrieve all RSVPs (requires admin role)
const handleGet: APIRoute = async ({ request }) => {
  const headers = {
    "Content-Type": "application/json",
  };

  try {
    const FILE_KEY = "rsvp.csv";
    const fileContent = await getS3Data<string>(FILE_KEY);
    
    if (!fileContent) {
      return new Response(JSON.stringify([]), {
        status: 200,
        headers,
      });
    }

    const rsvpList = csvToObjects(fileContent);
    return new Response(JSON.stringify(rsvpList), {
      status: 200,
      headers,
    });
  } catch (error) {
    console.error("Error retrieving RSVPs:", error);
    return new Response(
      JSON.stringify({
        success: false,
        error: "Failed to retrieve RSVPs",
      }),
      {
        status: 500,
        headers,
      }
    );
  }
};

// POST: Submit a new RSVP (requires guest role)
const handlePost: APIRoute = async ({ request }) => {
  const headers = {
    "Content-Type": "application/json",
  };

  try {
    const FILE_KEY = "rsvp.csv";
    const rawData = await request.json();

    // Validate required fields
    if (!rawData.name) {
      return new Response(
        JSON.stringify({
          success: false,
          error: "Name is required",
        }),
        {
          status: 400,
          headers,
        }
      );
    }

    // Sanitize the input data
    const newRsvp = sanitizeRSVP({
      ...rawData,
      timestamp: rawData.timestamp || new Date().toISOString(),
    });

    let existingRsvps: RSVPItem[] = [];
    const fileContent = await getS3Data<string>(FILE_KEY);
    if (fileContent) {
      existingRsvps = csvToObjects(fileContent);
    }

    existingRsvps.push(newRsvp);

    const csvContent = objectsToCSV(existingRsvps);
    await putS3Data(FILE_KEY, csvContent, "text/csv");

    return new Response(JSON.stringify({ success: true }), {
      status: 200,
      headers,
    });
  } catch (error) {
    console.error("Error handling RSVP:", error);
    return new Response(
      JSON.stringify({
        success: false,
        error: error instanceof Error ? error.message : "Unknown error",
      }),
      {
        status: 500,
        headers,
      }
    );
  }
};

// DELETE: Delete an RSVP (requires admin role)
const handleDelete: APIRoute = async ({ request }) => {
  const headers = {
    "Content-Type": "application/json",
  };

  try {
    const FILE_KEY = "rsvp.csv";
    const { name, timestamp } = await request.json();

    if (!name || !timestamp) {
      return new Response(
        JSON.stringify({
          success: false,
          error: "Name and timestamp are required",
        }),
        {
          status: 400,
          headers,
        }
      );
    }

    const fileContent = await getS3Data<string>(FILE_KEY);
    if (!fileContent) {
      return new Response(
        JSON.stringify({
          success: false,
          error: "No RSVPs found",
        }),
        {
          status: 404,
          headers,
        }
      );
    }

    const rsvpList = csvToObjects(fileContent);
    
    // Find exact match by name and timestamp
    const updatedList = rsvpList.filter(
      (rsvp) => !(rsvp.name === name && rsvp.timestamp === timestamp)
    );

    if (updatedList.length === rsvpList.length) {
      return new Response(
        JSON.stringify({
          success: false,
          error: "RSVP not found",
        }),
        {
          status: 404,
          headers,
        }
      );
    }

    const csvContent = objectsToCSV(updatedList);
    await putS3Data(FILE_KEY, csvContent, "text/csv");

    return new Response(JSON.stringify({ success: true }), {
      status: 200,
      headers,
    });
  } catch (error) {
    console.error("Error deleting RSVP:", error);
    return new Response(
      JSON.stringify({
        success: false,
        error: error instanceof Error ? error.message : "Unknown error",
      }),
      {
        status: 500,
        headers,
      }
    );
  }
};

// Export protected routes
export const GET = createProtectedAPIRoute(handleGet, "admin");
export const POST = createProtectedAPIRoute(handlePost, "guest");
export const DELETE = createProtectedAPIRoute(handleDelete, "admin");
